/*:
 * @target MZ
 * @plugindesc ピクチャを扇状に並べるプラグイン
 * @author やっつけ
 * @help AlignHands.js
 * Version 1.00
 * 
 * ピクチャを扇状に並べるプラグインだよ
 * 配列変数の要素の数だけ扇状に並べるよ
 * プラグインパラメータで変数を二つ用意してね
 * それとは別に配列変数も用意してね
 * 指定したピクチャ番号から順番に並べていくからピクチャ番号は連番にしてね
 * 配列の中身が0の場合ピクチャの移動をしないよ
 * 
 * 雑に言うと変数の操作でオペランドにスクリプト選んで[1,1,1,1,1,1]って打って読ませると6枚並べるよ
 * [1,,1,1,,1]を読ませると2枚目と5枚目が歯抜けになるよ
 * 
 * あくまで並べるだけなので『あらかじめピクチャの表示をしておく必要がある』、
 * リストの操作はあるけど『画像を張り替える機能は無い』のでそこは手動でお願いします
 * 
 * 機能を追加したり特殊な処理を行うプラグインではなく、コモンイベントのような代物
 * どこかでボロが出るだろうけど他プラグインとの衝突はまずない
 * 
 * 利用について
 * 用途・改変・再配布・権利表示に制限なし
 * 
 * 生まれたばかりのプラグインです。
 * 応援して下さいね☆
 * @url https://www.yahoo.co.jp/
 * 
 * @param 引数用変数1
 * @desc 変数ID　
 * @default 
 * @type variable
 * 
 * @param 引数用変数2
 * @desc 変数ID　
 * @default 
 * @type variable
 * 
 * @command MoveCards
 * @text ピクチャ配置
 * @desc ピクチャを扇状に配置します。
 * 
 * @arg cardlist
 * @text カードリスト
 * @desc カードリスト情報が格納された配列変数
 * @default 
 * @type variable
 * 
 * @arg pictureid
 * @text ピクチャ番号
 * @desc 一枚目のピクチャ番号
 * @default 
 * @type number
 * @min 1
 * 
 * @arg centerX
 * @text 中心点X
 * @desc 中心点のX座標
 * @default 
 * @type number
 * 
 * @arg centerY
 * @text 中心点Y
 * @desc 中心点のy座標
 * @default 
 * @type number
 * 
 * @arg center_angle
 * @text 中心角
 * @desc 扇の中心角
 * @default 
 * @type number
 * 
 * @arg direction
 * @text 方向
 * @desc 弧の中点が向いている方向（角度　右を0として時計回りに360で一周）
 * @default 
 * @type number
 * 
 * @arg radius
 * @text 半径
 * @desc 扇の半径
 * @default 
 * @type number
 * 
 * @arg origin
 * @text 原点
 * @desc ピクチャの原点（0:左上 1:中央）
 * @default 0
 * @type number
 * @max 1
 * @min 0
 * 
 * @arg scaleX
 * @text 拡大率：幅
 * @desc ピクチャの幅の拡大率
 * @default 100
 * @type number
 * @min 0
 * 
 * @arg scaleY
 * @text 拡大率：高さ
 * @desc ピクチャの高さの拡大率
 * @default 100
 * @type number
 * @min 0
 * 
 * @arg opacity
 * @text 不透明度
 * @desc ピクチャの不透明度（0～255）
 * @default 255
 * @type number
 * @max 255
 * @min 0
 * 
 * @arg blendMode
 * @text 合成方法
 * @desc ピクチャの合成方法（0:通常 1:加算 2:乗算 3:スクリーン）
 * @default 0
 * @type number
 * @max 3
 * @min 0
 * 
 * @arg duration
 * @text 移動時間
 * @desc 
 * @default 60
 * @type number
 * @min 1
 * 
 * @command kangaeruno_medoi
 * @text カードリスト操作
 * @desc カードを追加する/消す/書き換える
 * 
 * @arg cardlist
 * @text カードリスト
 * @desc カードリスト情報が格納された配列変数
 * @default 
 * @type variable
 * 
 * @arg operation
 * @text 操作選択
 * @desc 対象のカードの操作を選ぶ
 * @default 
 * @type select
 * @option 追加する
 * @option 削除する
 * @option 書き換える
 * 
*/
(() => {
    const PN = 'AlignHands';
    const param = PluginManager.parameters(PN);
    const param1 = param['引数用変数1'];
    const param2 = param['引数用変数2'];

    PluginManager.registerCommand(PN, "MoveCards", function (args) {
        var cardlist = $gameVariables.value(args.cardlist)
        var pictureid=Number(args.pictureid);
        var center_angle = Number(args.center_angle);
        var radius = Number(args.radius);
        var x;
        var y;
        var rad;
        if (Array.isArray(cardlist)) {
            for (var i = 0; i < cardlist.length; i++) {
                rad = Number(args.direction) - center_angle / 2 + center_angle / (cardlist.length) * i;
                x = (radius * Math.cos(rad * Math.PI / 180)) + Number(args.centerX);
                y = (radius * Math.sin(rad * Math.PI / 180)) + Number(args.centerY);
                if (cardlist[i]) {
                    $gameScreen.picture(pictureid+i)._angle = rad + 90;
                    $gameScreen.movePicture(pictureid+i, Number(args.origin), x, y,Number(args.scaleX), Number(args.scaleY),Number(args.opacity), Number(args.blendMode), Number(args.duration));
                }
            };
        };
    });

    PluginManager.registerCommand(PN, "kangaeruno_medoi", function (args) {
        var operation =args.operation;
        var arr1=$gameVariables.value(param1);
        if (operation === "追加する") {
            $gameVariables.value(args.cardlist).splice($gameVariables.value(param1), 0, $gameVariables.value(param2));
        } else if (operation === "書き換える") {
            $gameVariables.value(args.cardlist).splice($gameVariables.value(param1), 1, $gameVariables.value(param2));
        } else {
            $gameVariables.value(args.cardlist).splice(Number(arr1), 1);
        }


    });



})();