/*:
 * @target MZ
 * @plugindesc 虚無魔人（3枚目）
 * @author やっつけ
 * @help hidariue3.js
 * 戦闘グラフィックとマップスクロールを、四隅を基準にUIエリアに収める
 * UIエリアはデフォルトの816*624、画面の大きさはそれ以上を想定
 * あんまり極端な値にしなければある程度は大丈夫だと思う
 * UIエリア外の虚無空間はピクチャとかで隠して
 * 公式プラグインのExtraImage.jsや場所移動時にコモンイベント起動するプラグインとかと併用するといいかも
 * 
 * ウィンドウは個別設定めんどいから全部ずらした
 * 
 * 公式プラグインExtraImage.js使えばいらないけど、戦闘開始時虚無空間に画像を敷く機能も用意
 * アドレス例　：img/tilesets/
 * ファイル名例：World_C
 * 
 * プラグインコマンドで寄せ方向の変更ができるように
 * コマンド実行後に場所移動でマップ切り替えを行うと寄せ方向が切り替わる
 * 
 * ※注意点
 * ・「中央」はプラグイン入れない状態と同じなので挙動が他と異なる
 * ・戦闘背景は虚無空間にはみ出ないようにしているので中心がかなりずれる
 * 　「右下」、「左下」だと顕著
 * 
 * 更新：
 * ・タイトルに戻る対応＆フロントビューの戦闘背景修正
 * ・セーブロード対応
 * 
 * 絶対どっかでボロが出るか他プラグインと衝突するけどおこらないでね
 * 
 * 利用について
 * 用途・改変・再配布・権利表示に制限なし
 * 
 * 生まれたばかりのプラグインです。
 * 応援して下さいね☆
 * @url https://www.yahoo.co.jp/
 * 
 * @param 寄せ方向
 * @desc 四隅のどこに寄せるか
 * @default 0
 * @type select
 * @option ┏左上┐
 * @value 0
 * @option ┌右上┓
 * @value 1
 * @option │中央│
 * @value 2
 * @option ┗左下┘　
 * @value 3
 * @option └右下┛
 * @value 4
 * @param ウィンドウずらし
 * @desc ウィンドウが全部寄せ方向に沿ってずれる　全部って言ったら全部
 * @default true
 * @type boolean
 * @param 戦闘画面外画像アドレス
 * @desc 戦闘中の虚無空間に表示する画像のアドレス
 * @default 
 * @type text
 * @param 戦闘画面外画像ファイル名
 * @desc 戦闘中の虚無空間に表示する画像のファイル名　表示しない場合は空欄
 * @default 
 * @type text
 * 
 * @param 方向保存変数
 * @desc 設定保存用の変数ID　方向変更を使うなら設定
 * @default 
 * @type variable
 * 
 * @command RESERVE
 * @text 寄せ方向変更予約
 * @desc マップ切り替え時に指定の方向に寄せるよう予約
 * @arg direction
 * @text 寄せ方向
 * @desc 四隅のどこに寄せるか
 * @default 2
 * @type select
 * @option ┏左上┐
 * @value 0
 * @option ┌右上┓
 * @value 1
 * @option │中央│
 * @value 2
 * @option ┗左下┘　
 * @value 3
 * @option └右下┛
 * @value 4
*/
(() => {
    const PN = 'hidariue3';
    const param = PluginManager.parameters(PN);
    const uindou = param['ウィンドウずらし'];
    const haikeiA = param['戦闘画面外画像アドレス'];
    const haikeiB = param['戦闘画面外画像ファイル名'];
    const DV = param['方向保存変数'];
    var kijun = param['寄せ方向'];
    var yoyaku = kijun;

    PluginManager.registerCommand(PN, "RESERVE", function (args) {
        yoyaku = args.direction
        if(Boolean(DV)){$gameVariables.setValue(DV, yoyaku);}
    });
    Game_Map.prototype.setup = function (mapId) {
        if (!$dataMap) {
            throw new Error("The map data is not available");
        }
        this._mapId = mapId;
        this._tilesetId = $dataMap.tilesetId;
        this._displayX = 0;
        this._displayY = 0;
        this.refereshVehicles();
        this.setupEvents();
        this.setupScroll();
        this.setupParallax();
        this.setupBattleback();
        this._needsRefresh = false;

        kijun = yoyaku;
    };


    /* ウィンドウを全部ずらす */
    Scene_Base.prototype.createWindowLayer = function () {
        this._windowLayer = new WindowLayer();
        if (uindou == 'true') {
            if (kijun == 0) {
                this._windowLayer.x = 4;
                this._windowLayer.y = 4;
            } else if (kijun == 1) {
                this._windowLayer.x = Graphics.width - Graphics.boxWidth - 4;
                this._windowLayer.y = 4;
            } else if (kijun == 2) {
                this._windowLayer.x = (Graphics.width - Graphics.boxWidth) / 2;
                this._windowLayer.y = (Graphics.height - Graphics.boxHeight) / 2;
            } else if (kijun == 3) {
                this._windowLayer.x = 4;
                this._windowLayer.y = Graphics.height - Graphics.boxHeight - 4;
            } else {
                this._windowLayer.x = Graphics.width - Graphics.boxWidth - 4;
                this._windowLayer.y = Graphics.height - Graphics.boxHeight - 4;
            };
        } else {
            this._windowLayer.x = (Graphics.width - Graphics.boxWidth) / 2;
            this._windowLayer.y = (Graphics.height - Graphics.boxHeight) / 2;
        };
        this.addChild(this._windowLayer);
    };

    /* バトルキャラクターのグラフィックをずらす */
    Spriteset_Battle.prototype.createBattleField = function () {
        const width = (Graphics.boxWidth + 8);
        const height = (Graphics.boxHeight + 8);
        var xx;
        var yy;
        if (kijun == 0) {
            xx = 4;
            yy = 4;
        } else if (kijun == 1) {
            xx = Graphics.width - Graphics.boxWidth - 4;
            yy = 4;
        } else if (kijun == 2) {
            xx = (Graphics.width - width) / 2;
            yy = (Graphics.height - height) / 2;
        } else if (kijun == 3) {
            xx = 4;
            yy = Graphics.height - Graphics.boxHeight - 4;
        } else {
            xx = Graphics.width - Graphics.boxWidth - 4;
            yy = Graphics.height - Graphics.boxHeight - 4;
        };
        const x = xx;
        const y = yy;
        this._battleField = new Sprite();
        this._battleField.setFrame(0, 0, width, height);
        this._battleField.x = x;
        this._battleField.y = y - this.battleFieldOffsetY();
        this._baseSprite.addChild(this._battleField);
        this._effectsContainer = this._battleField;
    };

    /* 虚無空間にはみ出さないように戦闘背景をずらす */
    Sprite_Battleback.prototype.adjustPosition = function () {
        if (kijun == 2) {
            this.width = Math.floor((1000 * Graphics.width) / 816);
            this.height = Math.floor((740 * Graphics.height) / 624);
        }else{
        this.width = Math.floor((1000 * (Graphics.boxWidth + 8)) / 816);
        this.height = Math.floor((740 * (Graphics.boxHeight + 8)) / 624);
        };
        if (kijun == 0 || kijun == 3) {
            this.x = ((Graphics.boxWidth + 8) - this.width);
        } else if (kijun == 2) {
            this.x = (Graphics.width - this.width) / 2;
        } else {
            this.x = (Graphics.width - Graphics.boxWidth - 8);
        };
        if ($gameSystem.isSideView()) {
            if (kijun == 0 || kijun == 1) {
                this.y = (Graphics.boxHeight + 8) - this.height;
            } else if (kijun == 2) {
                this.y = Graphics.height - this.height;
            } else {
                this.y = (Graphics.height - Graphics.boxHeight - 8);
            };
        } else {{
            if (kijun == 0 || kijun == 1) {
                this.y = 0;
            } else if (kijun == 2) {
                this.y = Graphics.height - this.height;
            } else {
                this.y = (Graphics.height - Graphics.boxHeight - 8);
            };
        };
        };
        const ratioX = this.width / this.bitmap.width;
        const ratioY = this.height / this.bitmap.height;
        const scale = Math.max(ratioX, ratioY, 1.0);
        this.scale.x = scale;
        this.scale.y = scale;
    };

    /* 戦闘画面外の虚無空間に画像を貼る  */
    Spriteset_Battle.prototype.createBackground = function () {
        this._backgroundFilter = new PIXI.filters.BlurFilter();
        this._backgroundSprite = new Sprite();

        if (haikeiB) {
            this._backgroundSprite.bitmap = ImageManager.loadBitmap(haikeiA, haikeiB);
        } else {
            this._backgroundSprite.bitmap = SceneManager.backgroundBitmap();
            this._backgroundSprite.filters = [this._backgroundFilter];
        };
        this._baseSprite.addChild(this._backgroundSprite);
    };

    /* マップをずらす */
    /* マスを数える関数の定義 */
    Game_Map.prototype.screenTileXX = function () {
        return Math.round(((Graphics.boxWidth + 8) / this.tileWidth()) * 16) / 16;
    };
    Game_Map.prototype.screenTileYY = function () {
        return Math.round(((Graphics.boxHeight + 8) / this.tileHeight()) * 16) / 16;
    };
    /* カメラ位置ずらし */
    Game_Player.prototype.centerX = function () {
        if (kijun == 0 || kijun == 3) {
            return ($gameMap.screenTileXX() - 1) / 2;
        } else if (kijun == 2) {
            return ($gameMap.screenTileX() - 1) / 2;
        } else {
            return (($gameMap.screenTileX() + $gameMap.screenTileXX() * 0.5) - 1) / 2;
        };
    };
    Game_Player.prototype.centerY = function () {
        if (kijun == 0 || kijun == 1) {
            return ($gameMap.screenTileYY() - 1) / 2;
        } else if (kijun == 2) {
            return ($gameMap.screenTileY() - 1) / 2;
        } else {
            return ($gameMap.screenTileY() + $gameMap.screenTileYY() * 0.5 - 1) / 2;
        };

    };
    /* マップ呼び出し時のずらし */
    Game_Map.prototype.setDisplayPos = function (x, y) {
        if (this.isLoopHorizontal()) {
            this._displayX = x.mod(this.width());
            this._parallaxX = x;
        } else {
            const endX = this.width() - this.screenTileXX();
            if (kijun == 0 || kijun == 3) {
                this._displayX = (endX < 0 ? endX / 2 : x.clamp(0, endX));
            } else if (kijun == 2) {
                this._displayX = ((this.width() - this.screenTileX()) < 0 ? (this.width() - this.screenTileX()) / 2 : x.clamp(0, (this.width() - this.screenTileX())));
            } else {
                this._displayX = (endX < 0 ? endX / 2 : x.clamp(0, endX) + this.screenTileXX() - this.screenTileX());
            };

            this._parallaxX = this._displayX;
        }
        if (this.isLoopVertical()) {
            this._displayY = y.mod(this.height());
            this._parallaxY = y;
        } else {
            const endY = this.height() - this.screenTileYY();
            if (kijun == 0 || kijun == 1) {
                this._displayY = endY < 0 ? endY / 2 : y.clamp(0, endY);
            } else if (kijun == 2) {
                this._displayY = (this.height() - this.screenTileY()) < 0 ? (this.height() - this.screenTileY()) / 2 : y.clamp(0, (this.height() - this.screenTileY()));
            } else {
                this._displayY = endY < 0 ? endY / 2 : y.clamp(0, endY) + this.screenTileYY() - this.screenTileY();
            };

            this._parallaxY = this._displayY;
        }
    };
    /* スクロールの調整 */
    Game_Map.prototype.scrollDown = function (distance) {
        if (this.isLoopVertical()) {
            this._displayY += distance;
            this._displayY %= $dataMap.height;
            if (this._parallaxLoopY) {
                this._parallaxY += distance;
            }
        } else if (kijun == 2) {
            if (this.height() >= this.screenTileY()) {
                const lastY = this._displayY;
                this._displayY = Math.min(
                    this._displayY + distance,
                    this.height() - this.screenTileY()
                );
                this._parallaxY += this._displayY - lastY;
            }
        } else {
            if (this.height() >= this.screenTileYY()) {
                const lastY = this._displayY;
                if (kijun == 0 || kijun == 1) {
                    this._displayY = Math.min(
                        this._displayY + distance,
                        this.height() - this.screenTileYY()
                    );
                } else {
                    this._displayY = Math.min(
                        this._displayY + distance,
                        this.height() - this.screenTileY()
                    );
                };

                this._parallaxY += this._displayY - lastY;
            }
        }
    };
    Game_Map.prototype.scrollLeft = function (distance) {
        if (this.isLoopHorizontal()) {
            this._displayX += $dataMap.width - distance;
            this._displayX %= $dataMap.width;
            if (this._parallaxLoopX) {
                this._parallaxX -= distance;
            }
        } else if (kijun == 2) {
            if (this.width() >= this.screenTileX()) {
                const lastX = this._displayX;
                this._displayX = Math.max(this._displayX - distance, 0);
                this._parallaxX += this._displayX - lastX;
            }
        } else {
            if (this.width() >= this.screenTileXX()) {
                const lastX = this._displayX;
                if (kijun == 0 || kijun == 3) {
                    this._displayX = Math.max(this._displayX - distance, 0);
                } else {
                    this._displayX = Math.max(this._displayX - distance, this.screenTileXX() - this.screenTileX());
                };
                this._parallaxX += this._displayX - lastX;
            }
        }
    };
    Game_Map.prototype.scrollRight = function (distance) {
        if (this.isLoopHorizontal()) {
            this._displayX += distance;
            this._displayX %= $dataMap.width;
            if (this._parallaxLoopX) {
                this._parallaxX += distance;
            }
        } else if (kijun == 2) {
            if (this.width() >= this.screenTileX()) {
                const lastX = this._displayX;
                this._displayX = Math.min(
                    this._displayX + distance,
                    this.width() - this.screenTileX()
                );
                this._parallaxX += this._displayX - lastX;
            }
        } else {
            if (this.width() >= this.screenTileXX()) {
                const lastX = this._displayX;
                if (kijun == 0 || kijun == 3) {
                    this._displayX = Math.min(
                        this._displayX + distance,
                        this.width() - this.screenTileXX(),
                    );
                } else {
                    this._displayX = Math.min(
                        this._displayX + distance,
                        this.width() - this.screenTileX(),
                    );
                };
                this._parallaxX += this._displayX - lastX;
            }
        }
    };
    Game_Map.prototype.scrollUp = function (distance) {
        if (this.isLoopVertical()) {
            this._displayY += $dataMap.height - distance;
            this._displayY %= $dataMap.height;
            if (this._parallaxLoopY) {
                this._parallaxY -= distance;
            }
        } else if (kijun == 2) {
            if (this.height() >= this.screenTileY()) {
                const lastY = this._displayY;
                this._displayY = Math.max(this._displayY - distance, 0);
                this._parallaxY += this._displayY - lastY;
            }
        } else {
            if (this.height() >= this.screenTileYY()) {
                const lastY = this._displayY;
                if (kijun == 0 || kijun == 1) {
                    this._displayY = Math.max(this._displayY - distance, 0);
                } else {
                    this._displayY = Math.max(this._displayY - distance, this.screenTileYY() - this.screenTileY());
                };


                this._parallaxY += this._displayY - lastY;
            }
        }
    };
    //タイトルに戻った際設定を初期化
    const _Scene_Title_create = Scene_Title.prototype.create;
    Scene_Title.prototype.create = function() {
        kijun = param['寄せ方向'];
        _Scene_Title_create.apply(this, arguments);
    };
//ニューゲーム時に設定を変数に格納
    const _DataManager_setupNewGame = DataManager.setupNewGame;
    DataManager.setupNewGame = function () {
        _DataManager_setupNewGame.apply(this, arguments);
        $gameVariables.setValue(DV, param['寄せ方向']);
    };
    
//ロード時の設定読み込み
const _DataManager_loadGame = DataManager.loadGame;
DataManager.loadGame = function(savefileId) {
    return _DataManager_loadGame.apply(this, arguments).then(()=> {
        kijun=$gameVariables.value(DV)
        yoyaku=kijun
    })
};
})();