/*:
 * @target MZ
 * @plugindesc 虚無魔人（4枚目）
 * @author やっつけ
 * @help hidariue4.js
 * 戦闘グラフィックとマップスクロールを、四隅を基準にUIエリアに収める
 * UIエリアはデフォルトの816*624、画面の大きさはそれ以上での想定
 * あんまり極端に小さいor大きい値にしなければ破綻はしないと思う
 * UIエリア外の虚無空間はピクチャとかで隠して
 * 公式プラグインのExtraImage.jsや特定のタイミングでコモンイベント起動するプラグインとかと併用するといいかも
 * 
 * 設定保存用に変数を1つ専有するので設定してね
 * 
 * ウィンドウは個別設定めんどいから全部ずらした
 * 
 * プラグインコマンドで寄せ方向の変更ができるように
 * コマンド実行後に場所移動でマップ切り替えを行うと寄せ方向が切り替わる
 * 
 * プラグイン無効化とUIエリアサイズを画面サイズと同じにする機能を追加
 * プラグインコマンドで変更可能
 * 後者を使う場合前者も併用推奨
 * 
 * ※注意点
 * ・戦闘背景は虚無空間にはみ出ないようにしているだけなので中心がかなりずれる
 * 　「右下」、「左下」だと顕著
 * 
 * 更新：
 * ・戦闘画面背景に画像を張る機能を削除、パラメータや説明文を整備(更新前との互換性はない)
 * ・UIエリアサイズ最大時にウィンドウも画面いっぱいまで広がっていたので数ドット狭くなるよう修正
 * 
 * 絶対どっかでボロが出るか他プラグインと衝突するけどおこらないでね
 * 
 * 利用について
 * 用途・改変・再配布・権利表示に制限なし
 * 
 * 生まれたばかりのプラグインです。
 * 応援して下さいね☆
 * @url https://www.yahoo.co.jp/
 * 
 * @param 設定保存変数
 * @desc 設定保存用の変数ID　
 * @default 
 * @type variable
 * @param 画面寄せ方向
 * @desc 四隅のどこに寄せるか
 * @default 0
 * @type select
 * @option ┏左上┐
 * @value 0
 * @option ┌右上┓
 * @value 1
 * @option ┗左下┘　
 * @value 2
 * @option └右下┛
 * @value 3
 * @param 画面寄せ無効化
 * @desc 画面寄せを無効化した状態で起動する
 * @default false
 * @type boolean
 * @param UIエリア最大化
 * @desc UIエリアサイズを画面サイズに合わせた状態で起動する
 * @default false
 * @type boolean
 * 
 * @command RESERVE
 * @text 画面寄せ方向変更
 * @desc 指定の方向に寄せる（マップ切り替えで適用）
 * @arg direction
 * @text 画面寄せ方向
 * @desc 四隅のどこに寄せるか
 * @default 2
 * @type select
 * @option ┏左上┐
 * @value 0
 * @option ┌右上┓
 * @value 1
 * @option ┗左下┘　
 * @value 2
 * @option └右下┛
 * @value 3
 * 
 * @command DISABLE
 * @text 画面寄せ無効化
 * @desc 画面寄せを無効にする（マップ切り替えで適用）
 * @command ENABLE
 * @text 画面寄せ無効化の解除
 * @desc 画面寄せ無効化の解除（マップ切り替えで適用）
 * @command UIMAX
 * @text UIエリア最大化
 * @desc UIエリアを画面サイズに合わせる（マップ切り替えで適用）
 * @command UImodosu
 * @text UIエリア最大化の解除
 * @desc UIエリアを元に戻す（マップ切り替えで適用）
*/
(() => {
    const PN = 'hidariue4';
    const param = PluginManager.parameters(PN);
    const DV = param['設定保存変数'];
    var kijun = param['画面寄せ方向'];
    var yoyaku = kijun;
    var mukou = param['画面寄せ無効化'] === "true";
    var yoyaku_M = mukou;
    var saidaika = param['UIエリア最大化'] === "true";
    var yoyaku_S = saidaika;

    //画面とUIエリアのサイズ設定を取得
    var hidariue_width;
    var hidariue_boxWidth;
    var hidariue_height;
    var hidariue_boxHeight;
    const _Scene_Boot_adjustBoxSize = Scene_Boot.prototype.adjustBoxSize;
    Scene_Boot.prototype.adjustBoxSize = function () {
        _Scene_Boot_adjustBoxSize.apply(this, arguments);
        hidariue_width = Graphics.width;
        hidariue_boxWidth = Graphics.boxWidth;
        hidariue_height = Graphics.height;
        hidariue_boxHeight = Graphics.boxHeight;
    };

    PluginManager.registerCommand(PN, "RESERVE", function (args) {
        yoyaku = args.direction
        if (Boolean(DV)) { $gameVariables.setValue(DV, yoyaku); }
    });
    PluginManager.registerCommand(PN, "DISABLE", function (args) {
        yoyaku_M = true;
        if (Boolean(DV)) { $gameVariables.setValue(DV, $gameVariables.value(DV) | 4); }
    });
    PluginManager.registerCommand(PN, "ENABLE", function (args) {
        yoyaku_M = false;
        if (Boolean(DV)) { $gameVariables.setValue(DV, ($gameVariables.value(DV) | 4) ^ 4); }
    });
    PluginManager.registerCommand(PN, "UIMAX", function (args) {
        yoyaku_S = true;
        if (Boolean(DV)) { $gameVariables.setValue(DV, $gameVariables.value(DV) | 8); }
        UI_Apply();
    });
    PluginManager.registerCommand(PN, "UImodosu", function (args) {
        yoyaku_S = false;
        if (Boolean(DV)) {
            $gameVariables.setValue(DV, ($gameVariables.value(DV) | 8) ^ 8);
        }
    });

    /* UIエリア切り替え関数 */
    function UI_Apply() {
        if (saidaika) {
            Graphics.boxWidth = hidariue_width-8;
            Graphics.boxHeight = hidariue_height-8;
        } else {
            Graphics.boxWidth = hidariue_boxWidth;
            Graphics.boxHeight = hidariue_boxHeight;
        };
    };
    /* マップ切り替えで適用　*/
    const _Game_Map_setup = Game_Map.prototype.setup;
    Game_Map.prototype.setup = function (mapId) {
        _Game_Map_setup.apply(this, arguments);

        kijun = yoyaku;
        mukou = yoyaku_M;
        saidaika = yoyaku_S;
        UI_Apply();
    };

    /* ウィンドウを全部ずらす */
    const _Scene_Base_createWindowLayer = Scene_Base.prototype.createWindowLayer;
    Scene_Base.prototype.createWindowLayer = function () {
        if (mukou) {
            _Scene_Base_createWindowLayer.apply(this, arguments);
        } else {
            this._windowLayer = new WindowLayer();
            this._windowLayer.x = (kijun & 1) * (Graphics.width - Graphics.boxWidth-8)+4;
            this._windowLayer.y = ((kijun & 2) >> 1) * (Graphics.height - Graphics.boxHeight-8)+4;
            this.addChild(this._windowLayer);
        };
    };

    /* バトルキャラクターのグラフィックをずらす */
    const _Spriteset_Battle_createBattleField = Spriteset_Battle.prototype.createBattleField;
    Spriteset_Battle.prototype.createBattleField = function () {
        if (Boolean(mukou)) {
            _Spriteset_Battle_createBattleField.apply(this, arguments);
        } else {
            const width = Graphics.boxWidth;
            const height = Graphics.boxHeight;
            const x = (kijun & 1) * (Graphics.width - Graphics.boxWidth);
            const y = ((kijun & 2) >> 1) * (Graphics.height - Graphics.boxHeight);
            this._battleField = new Sprite();
            this._battleField.setFrame(0, 0, width, height);
            this._battleField.x = x;
            this._battleField.y = y - this.battleFieldOffsetY();
            this._baseSprite.addChild(this._battleField);
            this._effectsContainer = this._battleField;
        };
    };

    /* 虚無空間にはみ出さないように戦闘背景をずらす */
    const _Sprite_Battleback_adjustPosition = Sprite_Battleback.prototype.adjustPosition;
    Sprite_Battleback.prototype.adjustPosition = function () {
        if (Boolean(mukou)) {
            _Sprite_Battleback_adjustPosition.apply(this, arguments);
        } else {
            this.width = Math.floor((1000 * (Graphics.boxWidth + 8)) / 816);
            this.height = Math.floor((740 * (Graphics.boxHeight + 8)) / 624);

            if (kijun == 0 || kijun == 2) {
                this.x = ((Graphics.boxWidth + 8) - this.width);
            } else {
                this.x = (Graphics.width - Graphics.boxWidth - 8);
            };
            if ($gameSystem.isSideView()) {
                if (kijun == 0 || kijun == 1) {
                    this.y = (Graphics.boxHeight + 8) - this.height;
                } else {
                    this.y = (Graphics.height - Graphics.boxHeight - 8);
                };
            } else {
                {
                    if (kijun == 0 || kijun == 1) {
                        this.y = 0;
                    } else {
                        this.y = (Graphics.height - Graphics.boxHeight - 8);
                    };
                };
            };
            const ratioX = this.width / this.bitmap.width;
            const ratioY = this.height / this.bitmap.height;
            const scale = Math.max(ratioX, ratioY, 1.0);
            this.scale.x = scale;
            this.scale.y = scale;
        };
    };

    /* マップをずらす */
    /* マスを数える関数の定義 */
    Game_Map.prototype.screenTileXX = function () {
        return Math.round(((Graphics.boxWidth + 8) / this.tileWidth()) * 16) / 16;
    };
    Game_Map.prototype.screenTileYY = function () {
        return Math.round(((Graphics.boxHeight + 8) / this.tileHeight()) * 16) / 16;
    };
    /* カメラ位置ずらし */
    const _Game_Player_centerX = Game_Player.prototype.centerX;
    Game_Player.prototype.centerX = function () {
        if (Boolean(mukou)) {
            return _Game_Player_centerX.apply(this, arguments);
        } else {
            if (kijun == 0 || kijun == 2) {
                return ($gameMap.screenTileXX() - 1) / 2;
            } else {
                return (($gameMap.screenTileX() + $gameMap.screenTileXX() * 0.5) - 1) / 2;
            };
        };
    };
    const _Game_Player_centerY = Game_Player.prototype.centerY;
    Game_Player.prototype.centerY = function () {
        if (Boolean(mukou)) {
            return _Game_Player_centerY.apply(this, arguments);
        } else {
            if (kijun == 0 || kijun == 1) {
                return ($gameMap.screenTileYY() - 1) / 2;
            } else {
                return ($gameMap.screenTileY() + $gameMap.screenTileYY() * 0.5 - 1) / 2;
            };
        };
    };
    /* マップ呼び出し時のずらし */
    const _Game_Map_setDisplayPos = Game_Map.prototype.setDisplayPos;
    Game_Map.prototype.setDisplayPos = function (x, y) {
        if (Boolean(mukou)) {
            _Game_Map_setDisplayPos.apply(this, arguments);
        } else {
            if (this.isLoopHorizontal()) {
                this._displayX = x.mod(this.width());
                this._parallaxX = x;
            } else {
                const endX = this.width() - this.screenTileXX();
                if (kijun == 0 || kijun == 2) {
                    this._displayX = (endX < 0 ? endX / 2 : x.clamp(0, endX));
                } else {
                    this._displayX = (endX < 0 ? endX / 2 : x.clamp(0, endX) + this.screenTileXX() - this.screenTileX());
                };

                this._parallaxX = this._displayX;
            }
            if (this.isLoopVertical()) {
                this._displayY = y.mod(this.height());
                this._parallaxY = y;
            } else {
                const endY = this.height() - this.screenTileYY();
                if (kijun == 0 || kijun == 1) {
                    this._displayY = endY < 0 ? endY / 2 : y.clamp(0, endY);
                } else {
                    this._displayY = endY < 0 ? endY / 2 : y.clamp(0, endY) + this.screenTileYY() - this.screenTileY();
                };

                this._parallaxY = this._displayY;
            };
        };
    };
    /* スクロールの調整 */
    const _Game_Map_scrollDown = Game_Map.prototype.scrollDown;
    Game_Map.prototype.scrollDown = function (distance) {
        if (Boolean(mukou)) {
            _Game_Map_scrollDown.apply(this, arguments);
        } else {
            if (this.isLoopVertical()) {
                this._displayY += distance;
                this._displayY %= $dataMap.height;
                if (this._parallaxLoopY) {
                    this._parallaxY += distance;
                }
            } else {
                if (this.height() >= this.screenTileYY()) {
                    const lastY = this._displayY;
                    if (kijun == 0 || kijun == 1) {
                        this._displayY = Math.min(
                            this._displayY + distance,
                            this.height() - this.screenTileYY()
                        );
                    } else {
                        this._displayY = Math.min(
                            this._displayY + distance,
                            this.height() - this.screenTileY()
                        );
                    };

                    this._parallaxY += this._displayY - lastY;
                }
            }
        };
    };
    const _Game_Map_scrollLeft = Game_Map.prototype.scrollLeft;
    Game_Map.prototype.scrollLeft = function (distance) {
        if (Boolean(mukou)) {
            _Game_Map_scrollLeft.apply(this, arguments);
        } else {
            if (this.isLoopHorizontal()) {
                this._displayX += $dataMap.width - distance;
                this._displayX %= $dataMap.width;
                if (this._parallaxLoopX) {
                    this._parallaxX -= distance;
                }
            } else {
                if (this.width() >= this.screenTileXX()) {
                    const lastX = this._displayX;
                    if (kijun == 0 || kijun == 2) {
                        this._displayX = Math.max(this._displayX - distance, 0);
                    } else {
                        this._displayX = Math.max(this._displayX - distance, this.screenTileXX() - this.screenTileX());
                    };
                    this._parallaxX += this._displayX - lastX;
                }
            }
        };
    };
    const _Game_Map_scrollRight = Game_Map.prototype.scrollRight;
    Game_Map.prototype.scrollRight = function (distance) {
        if (Boolean(mukou)) {
            _Game_Map_scrollRight.apply(this, arguments);
        } else {
            if (this.isLoopHorizontal()) {
                this._displayX += distance;
                this._displayX %= $dataMap.width;
                if (this._parallaxLoopX) {
                    this._parallaxX += distance;
                }
            } else {
                if (this.width() >= this.screenTileXX()) {
                    const lastX = this._displayX;
                    if (kijun == 0 || kijun == 2) {
                        this._displayX = Math.min(
                            this._displayX + distance,
                            this.width() - this.screenTileXX(),
                        );
                    } else {
                        this._displayX = Math.min(
                            this._displayX + distance,
                            this.width() - this.screenTileX(),
                        );
                    };
                    this._parallaxX += this._displayX - lastX;
                }
            }
        };
    };
    const _Game_Map_scrollUp = Game_Map.prototype.scrollUp;
    Game_Map.prototype.scrollUp = function (distance) {
        if (Boolean(mukou)) {
            _Game_Map_scrollUp.apply(this, arguments);
        } else {
            if (this.isLoopVertical()) {
                this._displayY += $dataMap.height - distance;
                this._displayY %= $dataMap.height;
                if (this._parallaxLoopY) {
                    this._parallaxY -= distance;
                }
            } else {
                if (this.height() >= this.screenTileYY()) {
                    const lastY = this._displayY;
                    if (kijun == 0 || kijun == 1) {
                        this._displayY = Math.max(this._displayY - distance, 0);
                    } else {
                        this._displayY = Math.max(this._displayY - distance, this.screenTileYY() - this.screenTileY());
                    };


                    this._parallaxY += this._displayY - lastY;
                }
            }
        };
    };
    /* タイトルに戻った際設定を初期化 */
    const _Scene_Title_create = Scene_Title.prototype.create;
    Scene_Title.prototype.create = function () {
        kijun = param['画面寄せ方向'];
        yoyaku = kijun;
        mukou = param['画面寄せ無効化'] === "true";
        yoyaku_M = mukou;
        saidaika = param['UIエリア最大化'] === "true";
        yoyaku_S = saidaika;
        UI_Apply();
        _Scene_Title_create.apply(this, arguments);
    };

    /* ニューゲーム時に設定を変数に格納 */
    const _DataManager_setupNewGame = DataManager.setupNewGame;
    DataManager.setupNewGame = function () {
        _DataManager_setupNewGame.apply(this, arguments);
        if (Boolean(DV)) { $gameVariables.setValue(DV, (param['画面寄せ方向'] & 3) + (param['画面寄せ無効化'] === "true") * 4 + (param['UIエリア最大化'] === "true") * 8); }

        UI_Apply();

    };

    /* ロードゲーム時の設定読み込み */
    const _DataManager_loadGame = DataManager.loadGame;
    DataManager.loadGame = function (savefileId) {
        return _DataManager_loadGame.apply(this, arguments).then(() => {
            kijun = $gameVariables.value(DV) & 3
            yoyaku = kijun
            mukou = ($gameVariables.value(DV) & 8) === 4;
            yoyaku_M = mukou;
            saidaika = ($gameVariables.value(DV) & 8) === 8;
            yoyaku_S = saidaika;
            UI_Apply();
        })
    };
})();