/*:
 * @target MZ
 * @plugindesc 虚無魔人（EX）
 * @author やっつけ
 * @help hidariueEX.js
 * hidariueでウィンドウの寄りをするかしないかをちょっと拡張
 * hidariueのどれかの下に置いてウィンドウずらしをONにしてね
 * 
 * プラグインコマンドで設定変更可能
 * マップのウィンドウはマップ切り替えがされるまで適用されないよ
 * タイトル画面では初期設定に戻るよ
 * 
 * 設定保存用に変数を1つ専有するので設定してね
 * hidariue3以降で設定が必要な設定保存用変数とは別だよ
 * 
 * 絶対どっかでボロが出るか他プラグインと衝突するけどおこらないでね
 * 
 * 利用について
 * 用途・改変・再配布・権利表示に制限なし
 * 
 * 生まれたばかりのプラグインです。
 * 応援して下さいね☆
 * @url https://www.yahoo.co.jp/
 * 
 * @param タイトル
 * @desc タイトル画面のウィンドウをずらす
 * @default false
 * @type boolean
 * 
 * @param マップ
 * @desc マップのウィンドウをずらす
 * @default true
 * @type boolean
 * 
 * @param メニュー
 * @desc メニュー画面のウィンドウをずらす
 * @default true
 * @type boolean
 * 
 * @param バトル
 * @desc バトルのウィンドウをずらす
 * @default true
 * @type boolean
 * 
 * @param 切り替え設定変数
 * @desc 設定保存用の変数ID
 * @default 0
 * @type variable
 * 
 * @command Switch
 * @text ずらし切り替え
 * @desc ずらしのON/OFFを切り替える
 * @arg scene
 * @text 場面
 * @desc どの場面の設定を切り替えるか
 * @default 
 * @type select
 * @option マップ
 * @option メニュー
 * @option バトル　
 * @arg SwitchType
 * @text スイッチ動作
 * @desc ON/OFF/反転
 * @default 
 * @type select
 * @option ON（寄せ）
 * @option OFF（中央）
 * @option 反転

 * 
*/
(() => {
    const PN = 'hidariueEX';
    const param = PluginManager.parameters(PN);
    var W1 = param['タイトル'];
    var W2 = param['マップ'];
    var W3 = param['メニュー'];
    var W4 = param['バトル'];
    const SV = param['切り替え設定変数'];
    var bit1
    var bit2

    PluginManager.registerCommand(PN, "Switch", function (args) {
        bit1 = $gameVariables.value(SV)
        if (args.scene === 'マップ') {
            bit2 = 2
        } else if (args.scene === 'メニュー') {
            bit2 = 4
        } else if (args.scene === 'バトル') {
            bit2 = 8
        }
        if (args.SwitchType === 'ON（寄せ）') {
            $gameVariables.setValue(SV, bit1 | bit2);
        } else if (args.SwitchType === 'OFF（中央）') {
            $gameVariables.setValue(SV, (bit1 | bit2) ^ bit2);
        } else if (args.SwitchType === '反転') {
            $gameVariables.setValue(SV, bit1 ^ bit2);
        }
        yomikomi();
    });

    function yomikomi() {
        if (Boolean($gameVariables.value(SV) & 2)) {
            W2 = "true";
        } else {
            W2 = "false";
        };
        if (Boolean($gameVariables.value(SV) & 4)) {
            W3 = "true";
        } else {
            W3 = "false";
        };
        if (Boolean($gameVariables.value(SV) & 8)) {
            W4 = "true";
        } else {
            W4 = "false";
        };
    };
    function kakidashi() {
        bit1 = $gameVariables.value(SV)
        if (W2 === "true") {
            $gameVariables.setValue(SV, $gameVariables.value(SV) | 2);
        } else {
            $gameVariables.setValue(SV, ($gameVariables.value(SV) | 2) ^ 2);
        };
        if (W3 === "true") {
            $gameVariables.setValue(SV, $gameVariables.value(SV) | 4);
        } else {
            $gameVariables.setValue(SV, ($gameVariables.value(SV) | 4) ^ 4);
        };
        if (W4 === "true") {
            $gameVariables.setValue(SV, $gameVariables.value(SV) | 8);
        } else {
            $gameVariables.setValue(SV, ($gameVariables.value(SV) | 8) ^ 8);
        };
    };

    Scene_Base.prototype.createWindowLayer_org = function () {
        this._windowLayer = new WindowLayer();
        this._windowLayer.x = (Graphics.width - Graphics.boxWidth) / 2;
        this._windowLayer.y = (Graphics.height - Graphics.boxHeight) / 2;
        this.addChild(this._windowLayer);
    };

    Scene_Title.prototype.create = function () {
        Scene_Base.prototype.create.call(this);
        this.createBackground();
        this.createForeground();
        if (W1 === "true") {
            this.createWindowLayer();
        } else {
            this.createWindowLayer_org();
        };
        this.createCommandWindow();
    };
    Scene_Map.prototype.createDisplayObjects = function () {
        this.createSpriteset();
        if (W2 === "true") {
            this.createWindowLayer();
        } else {
            this.createWindowLayer_org();
        };
        this.createAllWindows();
        this.createButtons();
    };
    Scene_MenuBase.prototype.create = function () {
        Scene_Base.prototype.create.call(this);
        this.createBackground();
        this.updateActor();
        if (W3 === "true") {
            this.createWindowLayer();
        } else {
            this.createWindowLayer_org();
        };
        this.createButtons();
    };
    Scene_Battle.prototype.createDisplayObjects = function () {
        this.createSpriteset();
        if (W4 === "true") {
            this.createWindowLayer();
        } else {
            this.createWindowLayer_org();
        };
        this.createAllWindows();
        this.createButtons();
        BattleManager.setLogWindow(this._logWindow);
        BattleManager.setSpriteset(this._spriteset);
        this._logWindow.setSpriteset(this._spriteset);
    };
//タイトルに戻った際設定を初期化
    const _Scene_Title_start = Scene_Title.prototype.start;
    Scene_Title.prototype.start = function () {
        _Scene_Title_start.apply(this, arguments);
        W1 = param['タイトル'];
        W2 = param['マップ'];
        W3 = param['メニュー'];
        W4 = param['バトル'];
    };
    //ニューゲーム時に設定を変数に格納
    const _DataManager_setupNewGame = DataManager.setupNewGame;
    DataManager.setupNewGame = function () {
        _DataManager_setupNewGame.apply(this, arguments);
        kakidashi();
    };
    //ロード時に設定変数読み込み
    /*
    const _DataManager_loadGame = DataManager.loadGame;
    DataManager.loadGame = function(savefileId) {
     //   yomikomi();
        _DataManager_loadGame.apply(this, arguments);
    };
    */
    const _DataManager_loadGame = DataManager.loadGame;
    DataManager.loadGame = function(savefileId) {
        return _DataManager_loadGame.apply(this, arguments).then(()=> {
            yomikomi();
        })
    };



})();